#!/bin/sh

set -u

log_and_run(){
  local CMD=$1
  echo "${CMD}"
  ${CMD}
}

main(){
  mkdir -p /data/acme
  for IDX in $(printenv | grep -E '^ACME_\d+_EMAIL=' | sed -E 's/ACME_(\d+)_EMAIL.*/\1/'); do
    export ACME_IDX=${IDX}
    export ACME_CA=$(printenv ACME_${IDX}_CA)
    EMAIL=$(printenv ACME_${IDX}_EMAIL)
    CHALLENGE=$(printenv ACME_${IDX}_CHALLENGE)
  
    LEGO_OPTS="--path /data/acme --server ${ACME_CA} --email ${EMAIL} --accept-tos"
  
    if [ "${CHALLENGE}" = "http-01" ]; then
      LEGO_OPTS="${LEGO_OPTS} --http --http.port 127.0.0.1:${ACME_HTTP_PORT}"
    elif [ "${CHALLENGE}" = "dns-01" ]; then
      LEGO_OPTS="${LEGO_OPTS} --dns $(printenv ACME_${IDX}_DNS_PROVIDER)"
      export $(printenv ACME_${IDX}_DNS_KEY_ENV)=$(printenv ACME_${IDX}_DNS_KEY_VALUE)
      if printenv ACME_${IDX}_DNS_RESOLVERS; then
        LEGO_OPTS="${LEGO_OPTS} --dns.resolvers $(printenv ACME_${IDX}_DNS_RESOLVERS)"
      fi
    fi

    if printenv ACME_${IDX}_KEY_TYPE; then
      LEGO_OPTS="${LEGO_OPTS} --key-type $(printenv ACME_${IDX}_KEY_TYPE)"
    fi
  
    for CERT in $(printenv | grep -E "^ACME_${IDX}_CERT_\d+" | sed -E 's/([^=]+)=.*/\1/'); do
      # Use the first hostname as CN
      CN=$(printenv ${CERT} | sed -E 's/([^,]+).*/\1/')
      ACTION=run
      if [ -e /data/acme/certificates/${CN}.crt -a -e /data/acme/certificates/${CN}.key ]; then
        local CONF_DOMAIN=$(printenv ${CERT} | tr "," "\n" | sort | tr "\n" "," | sed -E 's/,$/\n/')
        local CUR_DOMAIN=$(openssl x509 -in /data/acme/certificates/${CN}.crt -noout -ext subjectAltName | tail -1 | sed -E 's/\s+DNS://g' | tr "," "\n" | sort | tr "\n" "," | sed -E 's/,$/\n/')
        if [ "${CUR_DOMAIN}" = "${CONF_DOMAIN}" ]; then
          echo "Certificate for ${CN} already exists, trying to renew (if needed)"
          ACTION=renew
        else
          echo "Certificate for ${CN} changed subjectAltName, trying to get a new cert"
        fi
      else
        echo "Certificate for ${CN} doesn't exist, trying to get it"
      fi
      log_and_run "lego ${LEGO_OPTS} --domains=$(printenv ${CERT}) ${ACTION} --${ACTION}-hook /usr/local/bin/upload-to-vault.sh"
    done

    CA=$(echo ${ACME_CA} | sed -E 's|^https?://([^/:]+).*|\1|')

    if ! vault kv get ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} > /dev/null 2>&1; then
      echo "Saving ACME account ${EMAIL} for ${ACME_CA} on vault"
      vault kv put ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} \
        metadata=@/data/acme/accounts/${CA}/${EMAIL}/account.json \
        key=@/data/acme/accounts/${CA}/${EMAIL}/keys/${EMAIL}.key
    else
      if [ "$(vault kv get -field metadata ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} | sha256sum | awk '{ print $1 }')" != "$(sha256sum /data/acme/accounts/${CA}/${EMAIL}/account.json | awk '{ print $1 }')" ]; then
        echo "Updating metadata of account ${EMAIL} for ${ACME_CA} on vault"
        vault kv patch ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} \
          metadata=@/data/acme/accounts/${CA}/${EMAIL}/account.json
      else
        echo "Metadata of account ${EMAIL} for ${ACME_CA} already in vault"
      fi
      if [ "$(vault kv get -field key ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} | sha256sum | awk '{ print $1 }')" != "$(sha256sum /data/acme/accounts/${CA}/${EMAIL}/keys/${EMAIL}.key | awk '{ print $1 }')" ]; then
        echo "Updating key of account ${EMAIL} for ${ACME_CA} on vault"
        vault kv patch ${ACME_KV_ACCOUNT_ROOT}/${CA}/${EMAIL} \
          key=@/data/acme/accounts/${CA}/${EMAIL}/keys/${EMAIL}.key
      else
        echo "Key of account ${EMAIL} for ${ACME_CA} already in vault"
      fi
    fi

  done
}

main
